@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2022
+# © Copyright EnterpriseDB UK Limited 2011-2023
 #
 # This file is part of Barman.
 #
@@ -37,7 +37,9 @@ from barman.backup_executor import (
     PassiveBackupExecutor,
     PostgresBackupExecutor,
     RsyncBackupExecutor,
+    SnapshotBackupExecutor,
 )
+from barman.cloud_providers import get_snapshot_interface_from_backup_info
 from barman.compression import CompressionManager
 from barman.config import BackupOptions
 from barman.exceptions import (
@@ -57,6 +59,7 @@ from barman.utils import (
     force_str,
     fsync_dir,
     fsync_file,
+    get_backup_info_from_name,
     human_readable_timedelta,
     pretty_size,
 )
@@ -88,6 +91,8 @@ class BackupManager(RemoteStatusMixin, KeepManagerMixin):
                 self.executor = PostgresBackupExecutor(self)
             elif self.config.backup_method == "local-rsync":
                 self.executor = RsyncBackupExecutor(self, local_mode=True)
+            elif self.config.backup_method == "snapshot":
+                self.executor = SnapshotBackupExecutor(self)
             else:
                 self.executor = RsyncBackupExecutor(self)
         except SshCommandException as e:
@@ -376,6 +381,20 @@ class BackupManager(RemoteStatusMixin, KeepManagerMixin):
         ids = sorted(available_backups.keys())
         return ids[0]
 
+    def get_backup_id_from_name(self, backup_name, status_filter=DEFAULT_STATUS_FILTER):
+        """
+        Get the id of the named backup, if it exists.
+
+        :param string backup_name: The name of the backup for which an ID should be
+            returned
+        :param tuple status_filter: The status of the backup to return.
+        :return string|None: ID of the backup
+        """
+        available_backups = self.get_available_backups(status_filter).values()
+        backup_info = get_backup_info_from_name(available_backups, backup_name)
+        if backup_info is not None:
+            return backup_info.backup_id
+
     @staticmethod
     def get_timelines_to_protect(remove_until, deleted_backup, available_backups):
         """
@@ -561,12 +580,13 @@ class BackupManager(RemoteStatusMixin, KeepManagerMixin):
 
         return True
 
-    def backup(self, wait=False, wait_timeout=None):
+    def backup(self, wait=False, wait_timeout=None, name=None):
         """
         Performs a backup for the server
 
         :param bool wait: wait for all the required WAL files to be archived
         :param int|None wait_timeout:
+        :param str|None name: the friendly name to be saved with this backup
         :return BackupInfo: the generated BackupInfo
         """
         _logger.debug("initialising backup information")
@@ -669,6 +689,9 @@ class BackupManager(RemoteStatusMixin, KeepManagerMixin):
             if backup_info:
                 backup_info.save()
 
+                # Set the backup friendly name if there is one
+                backup_info.backup_name = name
+
                 # Make sure we are not holding any PostgreSQL connection
                 # during the post-backup scripts
                 self.server.close()
@@ -730,7 +753,7 @@ class BackupManager(RemoteStatusMixin, KeepManagerMixin):
         # Delegate the recovery operation to a RecoveryExecutor object
 
         command = unix_command_factory(remote_command, self.server.path)
-        executor = recovery_executor_factory(self, command, backup_info.compression)
+        executor = recovery_executor_factory(self, command, backup_info)
         # Run the pre_recovery_script if present.
         script = HookScriptRunner(self, "recovery_script", "pre")
         script.env_from_recover(
@@ -831,7 +854,19 @@ class BackupManager(RemoteStatusMixin, KeepManagerMixin):
 
         :param barman.infofile.LocalBackupInfo backup: the backup to delete
         """
-        if backup.tablespaces:
+        # If this backup has snapshots then they should be deleted first.
+        if backup.snapshots_info:
+            _logger.debug(
+                "Deleting the following snapshots: %s"
+                % ", ".join(
+                    snapshot.identifier for snapshot in backup.snapshots_info.snapshots
+                )
+            )
+            snapshot_interface = get_snapshot_interface_from_backup_info(backup)
+            snapshot_interface.delete_snapshot_backup(backup)
+        # If this backup does *not* have snapshots then tablespaces are stored on the
+        # barman server so must be deleted.
+        elif backup.tablespaces:
             if backup.backup_version == 2:
                 tbs_dir = backup.get_basebackup_directory()
             else:
@@ -845,6 +880,9 @@ class BackupManager(RemoteStatusMixin, KeepManagerMixin):
                     )
                     shutil.rmtree(rm_dir)
 
+        # Whether a backup has snapshots or not, the data directory will always be
+        # present because this is where the backup_label is stored. It must therefore
+        # be deleted here.
         pg_data = backup.get_data_directory()
         if os.path.exists(pg_data):
             _logger.debug("Deleting PGDATA directory: %s" % pg_data)
