@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2022
+# © Copyright EnterpriseDB UK Limited 2011-2023
 #
 # This file is part of Barman.
 #
@@ -40,6 +40,7 @@ import dateutil.parser
 from distutils.version import LooseVersion as Version
 
 from barman import output, xlog
+from barman.cloud_providers import get_snapshot_interface_from_server_config
 from barman.command_wrappers import PgBaseBackup
 from barman.compression import get_pg_basebackup_compression
 from barman.config import BackupOptions
@@ -54,7 +55,7 @@ from barman.exceptions import (
     SshCommandException,
     FileNotFoundException,
 )
-from barman.fs import UnixLocalCommand, UnixRemoteCommand
+from barman.fs import UnixLocalCommand, UnixRemoteCommand, unix_command_factory
 from barman.infofile import BackupInfo
 from barman.postgres_plumbing import EXCLUDE_LIST, PGDATA_EXCLUDE_LIST
 from barman.remote_status import RemoteStatusMixin
@@ -80,6 +81,7 @@ class BackupExecutor(with_metaclass(ABCMeta, RemoteStatusMixin)):
 
         :param barman.backup.BackupManager backup_manager: the BackupManager
             assigned to the executor
+        :param str mode: The mode used by the executor for the backup.
         """
         super(BackupExecutor, self).__init__()
         self.backup_manager = backup_manager
@@ -615,6 +617,7 @@ class PostgresBackupExecutor(BackupExecutor):
             retry_handler=partial(self._retry_handler, dest_dirs),
             compression=self.backup_compression,
             err_handler=self._err_handler,
+            out_handler=PgBaseBackup.make_logging_handler(logging.INFO),
         )
 
         # Do the actual copy
@@ -730,12 +733,18 @@ class PostgresBackupExecutor(BackupExecutor):
         )
 
 
-class FsBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
+class ExternalBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
     """
-    Abstract base class file system level backup executors that
-    can operate remotely via SSH or locally:
+    Abstract base class for non-postgres backup executors.
+
+    An external backup executor is any backup executor which uses the
+    PostgreSQL low-level backup API to coordinate the backup.
+
+    Such executors can operate remotely via SSH or locally:
+
     - remote mode (default), operates via SSH
     - local mode, operates as the same user that Barman runs with
+
     It is also a factory for exclusive/concurrent backup strategy objects.
 
     Raises a SshCommandException if 'ssh_command' is not set and
@@ -748,11 +757,12 @@ class FsBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
 
         :param barman.backup.BackupManager backup_manager: the BackupManager
             assigned to the executor
+        :param str mode: The mode used by the executor for the backup.
         :param bool local_mode: if set to False (default), the class is able
             to operate on remote servers using SSH. Operates only locally
             if set to True.
         """
-        super(FsBackupExecutor, self).__init__(backup_manager, mode)
+        super(ExternalBackupExecutor, self).__init__(backup_manager, mode)
 
         # Set local/remote mode for copy
         self.local_mode = local_mode
@@ -827,7 +837,7 @@ class FsBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
         through the generic interface of a BackupExecutor. This implementation
         is responsible for performing a backup through a remote connection
         to the PostgreSQL server via Ssh. The specific set of instructions
-        depends on both the specific class that derives from FsBackupExecutor
+        depends on both the specific class that derives from ExternalBackupExecutor
         and the selected strategy (e.g. exclusive backup through Rsync).
 
         :param barman.infofile.LocalBackupInfo backup_info: backup information
@@ -886,7 +896,7 @@ class FsBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
 
     def _local_check(self, check_strategy):
         """
-        Specific checks for local mode of FsBackupExecutor (same user)
+        Specific checks for local mode of ExternalBackupExecutor (same user)
 
         :param CheckStrategy check_strategy: the strategy for the management
              of the results of the various checks
@@ -907,7 +917,7 @@ class FsBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
 
     def _remote_check(self, check_strategy):
         """
-        Specific checks for remote mode of FsBackupExecutor, via SSH.
+        Specific checks for remote mode of ExternalBackupExecutor, via SSH.
 
         :param CheckStrategy check_strategy: the strategy for the management
              of the results of the various checks
@@ -963,7 +973,7 @@ class FsBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
 
     def check(self, check_strategy):
         """
-        Perform additional checks for FsBackupExecutor, including
+        Perform additional checks for ExternalBackupExecutor, including
         Ssh connection (executing a 'true' command on the remote server)
         and specific checks for the given backup strategy.
 
@@ -987,7 +997,7 @@ class FsBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
 
     def status(self):
         """
-        Set additional status info for FsBackupExecutor using remote
+        Set additional status info for ExternalBackupExecutor using remote
         commands via Ssh, as well as those defined by the given
         backup strategy.
         """
@@ -1043,19 +1053,6 @@ class FsBackupExecutor(with_metaclass(ABCMeta, BackupExecutor)):
             _logger.warning("Error retrieving PostgreSQL status: %s", e)
         return remote_status
 
-    def _start_backup_copy_message(self, backup_info):
-        number_of_workers = self.config.parallel_jobs
-        via = "rsync/SSH"
-        if self.local_mode:
-            via = "local rsync"
-        message = "Starting backup copy via %s for %s" % (
-            via,
-            backup_info.backup_id,
-        )
-        if number_of_workers > 1:
-            message += " (%s jobs)" % number_of_workers
-        output.info(message)
-
 
 class PassiveBackupExecutor(BackupExecutor):
     """
@@ -1152,13 +1149,13 @@ class PassiveBackupExecutor(BackupExecutor):
         return "passive"
 
 
-class RsyncBackupExecutor(FsBackupExecutor):
+class RsyncBackupExecutor(ExternalBackupExecutor):
     """
     Concrete class for backup via Rsync+Ssh.
 
     It invokes PostgreSQL commands to start and stop the backup, depending
     on the defined strategy. Data files are copied using Rsync via Ssh.
-    It heavily relies on methods defined in the FsBackupExecutor class
+    It heavily relies on methods defined in the ExternalBackupExecutor class
     from which it derives.
     """
 
@@ -1387,6 +1384,286 @@ class RsyncBackupExecutor(FsBackupExecutor):
             return ":%s" % path
         return path
 
+    def _start_backup_copy_message(self, backup_info):
+        """
+        Output message for backup start.
+
+        :param barman.infofile.LocalBackupInfo backup_info: backup information
+        """
+        number_of_workers = self.config.parallel_jobs
+        via = "rsync/SSH"
+        if self.local_mode:
+            via = "local rsync"
+        message = "Starting backup copy via %s for %s" % (
+            via,
+            backup_info.backup_id,
+        )
+        if number_of_workers > 1:
+            message += " (%s jobs)" % number_of_workers
+        output.info(message)
+
+
+class SnapshotBackupExecutor(ExternalBackupExecutor):
+    """
+    Concrete class which uses cloud provider disk snapshots to create backups.
+
+    It invokes PostgreSQL commands to start and stop the backup, depending
+    on the defined strategy.
+
+    It heavily relies on methods defined in the ExternalBackupExecutor class
+    from which it derives.
+
+    No data files are copied and instead snapshots are created of the requested disks
+    using the cloud provider API (abstracted through a CloudSnapshotInterface).
+
+    As well as ensuring the backup happens via snapshot copy, this class also:
+
+      - Checks that the specified disks are attached to the named instance.
+      - Checks that the specified disks are mounted on the named instance.
+      - Records the mount points and options of each disk in the backup info.
+
+    Barman will still store the following files in its backup directory:
+
+      - The backup_label (for concurrent backups) which is written by the
+        LocalConcurrentBackupStrategy.
+      - The backup.info which is written by the BackupManager responsible for
+        instantiating this class.
+    """
+
+    def __init__(self, backup_manager):
+        """
+        Constructor for the SnapshotBackupExecutor
+
+        :param barman.backup.BackupManager backup_manager: the BackupManager
+            assigned to the strategy
+        """
+        super(SnapshotBackupExecutor, self).__init__(backup_manager, "snapshot")
+        self.snapshot_instance = self.config.snapshot_instance
+        self.snapshot_zone = self.config.snapshot_zone
+        self.snapshot_disks = self.config.snapshot_disks
+        self.validate_configuration()
+        try:
+            self.snapshot_interface = get_snapshot_interface_from_server_config(
+                self.config
+            )
+        except Exception as exc:
+            self.server.config.update_msg_list_and_disable_server(
+                "Error initialising snapshot provider %s: %s"
+                % (self.config.snapshot_provider, exc)
+            )
+
+    def validate_configuration(self):
+        """Verify configuration is valid for a snapshot backup."""
+        excluded_config = (
+            "backup_compression",
+            "bandwidth_limit",
+            "network_compression",
+            "tablespace_bandwidth_limit",
+        )
+        for config_var in excluded_config:
+            if getattr(self.server.config, config_var):
+                self.server.config.update_msg_list_and_disable_server(
+                    "%s option is not supported by snapshot backup_method" % config_var
+                )
+
+        if self.config.reuse_backup in ("copy", "link"):
+            self.server.config.update_msg_list_and_disable_server(
+                "reuse_backup option is not supported by snapshot backup_method"
+            )
+
+        required_config = (
+            "snapshot_disks",
+            "snapshot_instance",
+            "snapshot_provider",
+            "snapshot_zone",
+        )
+        for config_var in required_config:
+            if not getattr(self.server.config, config_var):
+                self.server.config.update_msg_list_and_disable_server(
+                    "%s option is required by snapshot backup_method" % config_var
+                )
+
+    @staticmethod
+    def add_mount_data_to_backup_info(backup_info, remote_cmd):
+        """
+        Adds the mount point and mount options for each disk to the backup info.
+
+        Each disk for which a snapshot was created during the backup is attached to the
+        instance at a given device path. This function uses the `findmnt` Unix command
+        to retrieve the mount point and mount options for each of those devices. The
+        mount data for each disk is then added to the metadata for the corresponding
+        snapshot in the backup info.
+
+        :param barman.infofile.LocalBackupInfo backup_info: Backup information.
+        :param UnixLocalCommand remote_cmd: Wrapper for local/remote commands.
+        """
+        for snapshot in backup_info.snapshots_info.snapshots:
+            mount_point, mount_options = remote_cmd.findmnt(snapshot.device)
+            if mount_point is None:
+                raise BackupException(
+                    "Could not find mount point for device %s" % snapshot.device
+                )
+            else:
+                snapshot.mount_point = mount_point
+                snapshot.mount_options = mount_options
+
+    def backup_copy(self, backup_info):
+        """
+        Perform the backup using cloud provider disk snapshots.
+
+        :param barman.infofile.LocalBackupInfo backup_info: Backup information.
+        """
+        # Create data dir so backup_label can be written
+        cmd = UnixLocalCommand(path=self.server.path)
+        cmd.create_dir_if_not_exists(backup_info.get_data_directory())
+
+        # Start the snapshot
+        self.copy_start_time = datetime.datetime.now()
+
+        self.snapshot_interface.take_snapshot_backup(
+            backup_info, self.snapshot_instance, self.snapshot_zone, self.snapshot_disks
+        )
+
+        self.copy_end_time = datetime.datetime.now()
+
+        # Gather additional metadata to assist recovery of snapshots
+        remote_cmd = UnixRemoteCommand(ssh_command=self.server.config.ssh_command)
+        self.add_mount_data_to_backup_info(backup_info, remote_cmd)
+
+        # Store statistics about the copy
+        copy_time = total_seconds(self.copy_end_time - self.copy_start_time)
+        backup_info.copy_stats = {
+            "copy_time": copy_time,
+            "total_time": copy_time,
+        }
+
+    @staticmethod
+    def find_missing_and_unmounted_disks(
+        cmd, snapshot_interface, snapshot_instance, snapshot_zone, snapshot_disks
+    ):
+        """
+        Checks for any disks listed in snapshot_disks which are not correctly attached
+        and mounted on the named instance and returns them as a tuple of two lists.
+
+        This is used for checking that the disks which are to be used as sources for
+        snapshots at backup time are attached and mounted on the instance to be backed
+        up.
+
+        :param UnixLocalCommand cmd: Wrapper for local/remote commands.
+        :param barman.cloud.CloudSnapshotInterface snapshot_interface: Interface for
+            taking snapshots and associated operations via cloud provider APIs.
+        :param str snapshot_instance: The name of the VM instance to which the disks
+            to be backed up are attached.
+        :param str snapshot_zone: The zone in which the snapshot disks and instance
+            reside.
+        :param list[str] snapshot_disks: A list containing the names of the disks for
+            which snapshots should be taken at backup time.
+        :rtype tuple[list[str],list[str]]
+        :return: A tuple where the first element is a list of all disks which are not
+            attached to the VM instance and the second element is a list of all disks
+            which are attached but not mounted.
+        """
+        attached_devices = snapshot_interface.get_attached_devices(
+            snapshot_instance, snapshot_zone
+        )
+        missing_disks = []
+        for disk in snapshot_disks:
+            if disk not in attached_devices.keys():
+                missing_disks.append(disk)
+
+        unmounted_disks = []
+        for disk in snapshot_disks:
+            try:
+                mount_point, _mount_options = cmd.findmnt(attached_devices[disk])
+            except KeyError:
+                # Ignore disks which were not attached
+                continue
+            if mount_point is None:
+                unmounted_disks.append(disk)
+
+        return missing_disks, unmounted_disks
+
+    def check(self, check_strategy):
+        """
+        Perform additional checks for SnapshotBackupExecutor, specifically:
+
+          - check that the VM instance for which snapshots should be taken exists
+          - check that the expected disks are attached to that instance
+          - check that the attached disks are mounted on the filesystem
+
+        :param CheckStrategy check_strategy: the strategy for the management
+             of the results of the various checks
+        """
+        super(SnapshotBackupExecutor, self).check(check_strategy)
+        if self.server.config.disabled:
+            # Skip checks if the server is not active
+            return
+        check_strategy.init_check("snapshot instance exists in zone")
+        if not self.snapshot_interface.instance_exists(
+            self.snapshot_instance, self.snapshot_zone
+        ):
+            check_strategy.result(
+                self.config.name,
+                False,
+                hint="cannot find compute instance %s in zone %s"
+                % (self.snapshot_instance, self.snapshot_zone),
+            )
+            return
+        else:
+            check_strategy.result(self.config.name, True)
+
+        check_strategy.init_check("snapshot disks attached to instance")
+        cmd = unix_command_factory(self.config.ssh_command, self.server.path)
+        missing_disks, unmounted_disks = self.find_missing_and_unmounted_disks(
+            cmd,
+            self.snapshot_interface,
+            self.snapshot_instance,
+            self.snapshot_zone,
+            self.snapshot_disks,
+        )
+
+        if len(missing_disks) > 0:
+            check_strategy.result(
+                self.config.name,
+                False,
+                hint="cannot find snapshot disks attached to instance %s: %s"
+                % (self.snapshot_instance, ", ".join(missing_disks)),
+            )
+        else:
+            check_strategy.result(self.config.name, True)
+
+        check_strategy.init_check("snapshot disks mounted on instance")
+        if len(unmounted_disks) > 0:
+            check_strategy.result(
+                self.config.name,
+                False,
+                hint="cannot find snapshot disks mounted on instance %s: %s"
+                % (self.snapshot_instance, ", ".join(unmounted_disks)),
+            )
+        else:
+            check_strategy.result(self.config.name, True)
+
+    def _start_backup_copy_message(self, backup_info):
+        """
+        Output message for backup start.
+
+        :param barman.infofile.LocalBackupInfo backup_info: Backup information.
+        """
+        output.info("Starting backup with disk snapshots for %s", backup_info.backup_id)
+
+    def _stop_backup_copy_message(self, backup_info):
+        """
+        Output message for backup end.
+
+        :param barman.infofile.LocalBackupInfo backup_info: Backup information.
+        """
+        output.info(
+            "Snapshot backup done (time: %s)",
+            human_readable_timedelta(
+                datetime.timedelta(seconds=backup_info.copy_stats["copy_time"])
+            ),
+        )
+
 
 class BackupStrategy(with_metaclass(ABCMeta, object)):
     """
@@ -1722,7 +1999,7 @@ class ExclusiveBackupStrategy(BackupStrategy):
     """
     Concrete class for exclusive backup strategy.
 
-    This strategy is for FsBackupExecutor only and is responsible for
+    This strategy is for ExternalBackupExecutor only and is responsible for
     coordinating Barman with PostgreSQL on standard physical backup
     operations (known as 'exclusive' backup), such as invoking
     pg_start_backup() and pg_stop_backup() on the master server.
@@ -1818,7 +2095,7 @@ class ConcurrentBackupStrategy(BackupStrategy):
 
     This strategy is responsible for coordinating Barman with PostgreSQL on
     concurrent physical backup operations through concurrent backup
-    PostgreSQL api or the pgespresso extension.
+    PostgreSQL api.
     """
 
     def __init__(self, postgres, server_name):
@@ -1835,26 +2112,26 @@ class ConcurrentBackupStrategy(BackupStrategy):
 
     def check(self, check_strategy):
         """
-        Perform additional checks for ConcurrentBackupStrategy
+        Checks that Postgres is at least minimal version
 
         :param CheckStrategy check_strategy: the strategy for the management
              of the results of the various checks
         """
-        check_strategy.init_check("pgespresso extension")
+        check_strategy.init_check("postgres minimal version")
         try:
-            # We execute this check only if the postgres connection is non None
-            # and the server version is lower than 9.6. On latest PostgreSQL
-            # there is a native API for concurrent backups.
-            if self.postgres and self.postgres.server_version < 90600:
-                if self.postgres.has_pgespresso:
-                    check_strategy.result(self.server_name, True)
-                else:
-                    check_strategy.result(
-                        self.server_name,
-                        False,
-                        hint="required for concurrent "
-                        "backups on PostgreSQL %s" % self.postgres.server_major_version,
-                    )
+            # We execute this check only if the postgres connection is not None
+            # to validate the server version matches at least minimal version
+            if self.postgres and not self.postgres.is_minimal_postgres_version():
+                check_strategy.result(
+                    self.server_name,
+                    False,
+                    hint="unsupported PostgresSQL version %s. Expecting %s or above."
+                    % (
+                        self.postgres.server_major_version,
+                        self.postgres.minimal_txt_version,
+                    ),
+                )
+
         except PostgresConnectionError:
             # Skip the check if the postgres connection doesn't work.
             # We assume that this error condition will be reported by
@@ -1874,15 +2151,12 @@ class ConcurrentBackupStrategy(BackupStrategy):
 
         label = "Barman backup %s %s" % (backup_info.server_name, backup_info.backup_id)
 
-        pg_version = self.postgres.server_version
-        if pg_version >= 90600:
-            # On 9.6+ execute native concurrent start backup
-            _logger.debug("Start of native concurrent backup")
-            self._concurrent_start_backup(backup_info, label)
-        else:
-            # On older Postgres use pgespresso
-            _logger.debug("Start of concurrent backup with pgespresso")
-            self._pgespresso_start_backup(backup_info, label)
+        if not self.postgres.is_minimal_postgres_version():
+            _logger.error("Postgres version not supported")
+            raise BackupException("Postgres version not supported")
+        # On 9.6+ execute native concurrent start backup
+        _logger.debug("Start of native concurrent backup")
+        self._concurrent_start_backup(backup_info, label)
 
     def stop_backup(self, backup_info):
         """
@@ -1890,20 +2164,22 @@ class ConcurrentBackupStrategy(BackupStrategy):
 
         :param barman.infofile.BackupInfo backup_info: backup information
         """
-        self.current_action = "issuing stop backup command"
-        pg_version = self.postgres.server_version
-        if pg_version >= 90600:
-            # On 9.6+ execute native concurrent stop backup
-            self.current_action += " (native concurrent)"
-            _logger.debug("Stop of native concurrent backup")
-            self._concurrent_stop_backup(backup_info)
-        else:
-            # On older Postgres use pgespresso
-            self.current_action += " (pgespresso)"
-            _logger.debug("Stop of concurrent backup with pgespresso")
-            self._pgespresso_stop_backup(backup_info)
+        self.current_action = "issuing stop backup command (native concurrent)"
+        if not self.postgres.is_minimal_postgres_version():
+            _logger.error(
+                "Postgres version not supported. Minimal version is %s"
+                % self.postgres.minimal_txt_version
+            )
+            raise BackupException("Postgres version not supported")
 
-        # Write backup_label retrieved from postgres connection
+        _logger.debug("Stop of native concurrent backup")
+        self._concurrent_stop_backup(backup_info)
+
+        # Update the current action in preparation for writing the backup label.
+        # NOTE: The actual writing of the backup label happens either in the
+        # specialization of this function in LocalConcurrentBackupStrategy
+        # or out-of-band in a CloudBackupUploader (when ConcurrentBackupStrategy
+        # is used directly when writing to an object store).
         self.current_action = "writing backup label"
 
         # Ask PostgreSQL to switch to another WAL file. This is needed
@@ -1915,35 +2191,6 @@ class ConcurrentBackupStrategy(BackupStrategy):
             # Skip switching XLOG if a standby server
             pass
 
-    def _pgespresso_start_backup(self, backup_info, label):
-        """
-        Start a concurrent backup using pgespresso
-
-        :param barman.infofile.BackupInfo backup_info: backup information
-        """
-        backup_info.set_attribute("status", BackupInfo.STARTED)
-        start_info = self.postgres.pgespresso_start_backup(label)
-        backup_info.set_attribute("backup_label", start_info["backup_label"])
-        self._backup_info_from_backup_label(backup_info)
-
-    def _pgespresso_stop_backup(self, backup_info):
-        """
-        Stop a concurrent backup using pgespresso
-
-        :param barman.infofile.BackupInfo backup_info: backup information
-        """
-        stop_info = self.postgres.pgespresso_stop_backup(backup_info.backup_label)
-        # Obtain a modifiable copy of stop_info object
-        stop_info = stop_info.copy()
-        # We don't know the exact backup stop location,
-        # so we include the whole segment.
-        stop_info["location"] = xlog.location_from_xlogfile_name_offset(
-            stop_info["end_wal"],
-            self.postgres.xlog_segment_size - 1,
-            self.postgres.xlog_segment_size,
-        )
-        self._backup_info_from_stop_location(backup_info, stop_info)
-
     def _concurrent_start_backup(self, backup_info, label):
         """
         Start a concurrent backup using the PostgreSQL 9.6
@@ -1973,9 +2220,9 @@ class LocalConcurrentBackupStrategy(ConcurrentBackupStrategy):
     """
     Concrete class for concurrent backup strategy writing data locally.
 
-    This strategy is for FsBackupExecutor only and is responsible for
+    This strategy is for ExternalBackupExecutor only and is responsible for
     coordinating Barman with PostgreSQL on concurrent physical backup
-    operations through the pgespresso extension.
+    operations through concurrent backup PostgreSQL api.
     """
 
     # noinspection PyMethodMayBeStatic
