@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2022
+# © Copyright EnterpriseDB UK Limited 2011-2023
 #
 # This file is part of Barman.
 #
@@ -48,6 +48,7 @@ from barman.infofile import BackupInfo, WalFileInfo
 from barman.server import Server
 from barman.utils import (
     BarmanEncoder,
+    check_backup_name,
     check_non_negative,
     check_positive,
     check_tli,
@@ -55,7 +56,9 @@ from barman.utils import (
     drop_privileges,
     force_str,
     get_log_levels,
+    is_backup_id,
     parse_log_level,
+    RESERVED_BACKUP_IDS,
     SHA256,
 )
 from barman.xlog import check_archive_usable
@@ -334,7 +337,7 @@ def backup_completer(prefix, parsed_args, **kwargs):
     for backup_id in sorted(backups, reverse=True):
         if backup_id.startswith(prefix):
             yield backup_id
-    for special_id in ("latest", "last", "oldest", "first", "last-failed"):
+    for special_id in RESERVED_BACKUP_IDS:
         if len(backups) > 0 and special_id.startswith(prefix):
             yield special_id
 
@@ -421,6 +424,14 @@ def backup_completer(prefix, parsed_args, **kwargs):
             default=None,
             type=check_non_negative,
         ),
+        argument(
+            "--name",
+            help="a name which can be used to reference this backup in barman "
+            "commands such as recover and delete",
+            dest="backup_name",
+            default=None,
+            type=check_backup_name,
+        ),
     ]
 )
 def backup(args):
@@ -452,7 +463,11 @@ def backup(args):
         if hasattr(args, "bwlimit"):
             server.config.bandwidth_limit = args.bwlimit
         with closing(server):
-            server.backup(wait=args.wait, wait_timeout=args.wait_timeout)
+            server.backup(
+                wait=args.wait,
+                wait_timeout=args.wait_timeout,
+                backup_name=args.backup_name,
+            )
     output.close_and_exit()
 
 
@@ -724,6 +739,14 @@ def rebuild_xlogdb(args):
                 "backup."
             ),
         ),
+        argument(
+            "--snapshot-recovery-instance",
+            help="Instance where the disks recovered from the snapshots are attached",
+        ),
+        argument(
+            "--snapshot-recovery-zone",
+            help="Zone containing the instance and disks for the snapshot recovery",
+        ),
     ]
 )
 def recover(args):
@@ -862,6 +885,47 @@ def recover(args):
             output.close_and_exit()
         server.config.network_compression = args.network_compression
 
+    if backup_id.snapshots_info is not None:
+        missing_args = []
+        if not args.snapshot_recovery_instance:
+            missing_args.append("--snapshot-recovery-instance")
+        if not args.snapshot_recovery_zone:
+            missing_args.append("--snapshot-recovery-zone")
+        if len(missing_args) > 0:
+            output.error(
+                "Backup %s is a snapshot backup and the following required arguments "
+                "have not been provided: %s",
+                backup_id.backup_id,
+                ", ".join(missing_args),
+            )
+            output.close_and_exit()
+        if tablespaces != {}:
+            output.error(
+                "Backup %s is a snapshot backup therefore tablespace relocation rules "
+                "cannot be used.",
+                backup_id.backup_id,
+            )
+            output.close_and_exit()
+        snapshot_kwargs = {
+            "recovery_instance": args.snapshot_recovery_instance,
+            "recovery_zone": args.snapshot_recovery_zone,
+        }
+    else:
+        unexpected_args = []
+        if args.snapshot_recovery_instance:
+            unexpected_args.append("--snapshot-recovery-instance")
+        if args.snapshot_recovery_zone:
+            unexpected_args.append("--snapshot-recovery-zone")
+        if len(unexpected_args) > 0:
+            output.error(
+                "Backup %s is not a snapshot backup but the following snapshot "
+                "arguments have been used: %s",
+                backup_id.backup_id,
+                ", ".join(unexpected_args),
+            )
+            output.close_and_exit()
+        snapshot_kwargs = {}
+
     with closing(server):
         try:
             server.recover(
@@ -878,6 +942,7 @@ def recover(args):
                 remote_command=args.remote_ssh_command,
                 target_action=getattr(args, "target_action", None),
                 standby_mode=getattr(args, "standby_mode", None),
+                **snapshot_kwargs
             )
         except RecoveryException as exc:
             output.error(force_str(exc))
@@ -1494,7 +1559,9 @@ def verify_backup(args):
     # Raises an error if wrong backup
     backup_info = parse_backup_id(server, args)
     # get backup path
-    output.info("Verifying backup %s on server %s" % (args.backup_id, args.server_name))
+    output.info(
+        "Verifying backup '%s' on server %s" % (args.backup_id, args.server_name)
+    )
 
     server.backup_manager.verify_backup(backup_info)
     output.close_and_exit()
@@ -1528,7 +1595,7 @@ def generate_manifest(args):
     backup_manifest.create_backup_manifest()
 
     output.info(
-        "Backup manifest for backup %s successfully generated for server %s"
+        "Backup manifest for backup '%s' successfully generated for server %s"
         % (args.backup_id, args.server_name)
     )
     output.close_and_exit()
@@ -1842,17 +1909,17 @@ def get_server_list(
         # When 'all' is used, it must be the only specified argument
         if args and len(args.server_name) != 1:
             output.error("You cannot use 'all' with other server names")
-        servers = available_servers
+        server_names = available_servers
     else:
         # Put servers in a set, so multiple occurrences are counted only once
-        servers = set(args.server_name)
+        server_names = set(args.server_name)
 
     # Loop through all the requested servers
-    for server in servers:
-        conf = barman.__config__.get_server(server)
+    for server_name in server_names:
+        conf = barman.__config__.get_server(server_name)
         if conf is None:
             # Unknown server
-            server_dict[server] = None
+            server_dict[server_name] = None
         else:
             server_object = Server(conf)
             # Skip inactive servers, if requested
@@ -1867,7 +1934,7 @@ def get_server_list(
             if skip_passive and server_object.passive_node:
                 output.info("Skipping passive server '%s'", conf.name)
                 continue
-            server_dict[server] = server_object
+            server_dict[server_name] = server_object
 
     return server_dict
 
@@ -1939,8 +2006,14 @@ def parse_backup_id(server, args):
         backup_id = server.get_first_backup_id()
     elif args.backup_id in ("last-failed"):
         backup_id = server.get_last_backup_id([BackupInfo.FAILED])
-    else:
+    elif is_backup_id(args.backup_id):
         backup_id = args.backup_id
+    else:
+        try:
+            backup_id = server.get_backup_id_from_name(args.backup_id)
+        except ValueError as exc:
+            output.error(str(exc))
+            output.close_and_exit()
     backup_info = server.get_backup(backup_id)
     if backup_info is None:
         output.error(
