@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2018-2022
+# © Copyright EnterpriseDB UK Limited 2018-2023
 #
 # This file is part of Barman.
 #
@@ -30,7 +30,10 @@ from barman.clients.cloud_cli import (
     OperationErrorExit,
 )
 from barman.cloud import CloudBackupCatalog, configure_logging
-from barman.cloud_providers import get_cloud_interface
+from barman.cloud_providers import (
+    get_cloud_interface,
+    get_snapshot_interface_from_backup_info,
+)
 from barman.exceptions import InvalidRetentionPolicy
 from barman.retention_policies import RetentionPolicyFactory
 from barman.utils import force_str
@@ -170,6 +173,16 @@ def _delete_backup(
     if not backup_info:
         logging.warning("Backup %s does not exist", backup_id)
         return
+    if backup_info.snapshots_info:
+        logging.debug(
+            "Will delete the following snapshots: %s",
+            ", ".join(
+                snapshot.identifier for snapshot in backup_info.snapshots_info.snapshots
+            ),
+        )
+        if not dry_run:
+            snapshot_interface = get_snapshot_interface_from_backup_info(backup_info)
+            snapshot_interface.delete_snapshot_backup(backup_info)
     objects_to_delete = _get_files_for_backup(catalog, backup_info)
     backup_info_path = os.path.join(
         catalog.prefix, backup_info.backup_id, "backup.info"
@@ -238,23 +251,21 @@ def main(args=None):
                     % catalog.unreadable_backups
                 )
                 raise OperationErrorExit()
-
             if config.backup_id:
+                backup_id = catalog.parse_backup_id(config.backup_id)
                 # Because we only care about one backup, skip the annotation cache
                 # because it is only helpful when dealing with multiple backups
-                if catalog.should_keep_backup(config.backup_id, use_cache=False):
+                if catalog.should_keep_backup(backup_id, use_cache=False):
                     logging.error(
                         "Skipping delete of backup %s for server %s "
                         "as it has a current keep request. If you really "
                         "want to delete this backup please remove the keep "
                         "and try again.",
-                        config.backup_id,
+                        backup_id,
                         config.server_name,
                     )
                     raise OperationErrorExit()
-                _delete_backup(
-                    cloud_interface, catalog, config.backup_id, config.dry_run
-                )
+                _delete_backup(cloud_interface, catalog, backup_id, config.dry_run)
             elif config.retention_policy:
                 try:
                     retention_policy = RetentionPolicyFactory.create(
