@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2018-2022
+# © Copyright EnterpriseDB UK Limited 2018-2023
 #
 # This file is part of Barman.
 #
@@ -15,6 +15,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with Barman.  If not, see <http://www.gnu.org/licenses/>.
+from abc import ABCMeta, abstractmethod
 import logging
 import os
 from contextlib import closing
@@ -25,10 +26,42 @@ from barman.clients.cloud_cli import (
     GeneralErrorExit,
     NetworkErrorExit,
     OperationErrorExit,
+    get_missing_attrs,
 )
 from barman.cloud import CloudBackupCatalog, configure_logging
-from barman.cloud_providers import get_cloud_interface
-from barman.utils import force_str
+from barman.cloud_providers import (
+    get_cloud_interface,
+    get_snapshot_interface_from_backup_info,
+)
+from barman.exceptions import ConfigurationException
+from barman.fs import UnixLocalCommand
+from barman.recovery_executor import SnapshotRecoveryExecutor
+from barman.utils import force_str, with_metaclass
+
+
+def _validate_config(config, backup_info):
+    """
+    Additional validation for config such as mutually inclusive options.
+
+    Raises a ConfigurationException if any options are missing or incompatible.
+
+    :param argparse.Namespace config: The backup options provided at the command line.
+    :param BackupInfo backup_info: The backup info for the backup to restore
+    """
+    if backup_info.snapshots_info:
+        missing_options = get_missing_attrs(
+            config, ("snapshot_recovery_instance", "snapshot_recovery_zone")
+        )
+        if len(missing_options) > 0:
+            raise ConfigurationException(
+                "Incomplete options for snapshot restore - missing: %s"
+                % ", ".join(missing_options)
+            )
+        if config.tablespace != []:
+            raise ConfigurationException(
+                "Backup %s is a snapshot backup therefore tablespace relocation rules "
+                "cannot be used." % backup_info.backup_id,
+            )
 
 
 def main(args=None):
@@ -41,21 +74,10 @@ def main(args=None):
     config = parse_arguments(args)
     configure_logging(config)
 
-    # Validate the destination directory before starting recovery
-    if os.path.exists(config.recovery_dir) and os.listdir(config.recovery_dir):
-        logging.error(
-            "Destination %s already exists and it is not empty", config.recovery_dir
-        )
-        raise OperationErrorExit()
-
     try:
         cloud_interface = get_cloud_interface(config)
 
         with closing(cloud_interface):
-            downloader = CloudBackupDownloader(
-                cloud_interface=cloud_interface, server_name=config.server_name
-            )
-
             if not cloud_interface.test_connectivity():
                 raise NetworkErrorExit()
             # If test is requested, just exit after connectivity test
@@ -66,11 +88,35 @@ def main(args=None):
                 logging.error("Bucket %s does not exist", cloud_interface.bucket_name)
                 raise OperationErrorExit()
 
-            downloader.download_backup(
-                config.backup_id,
-                config.recovery_dir,
-                tablespace_map(config.tablespace),
-            )
+            catalog = CloudBackupCatalog(cloud_interface, config.server_name)
+            backup_id = catalog.parse_backup_id(config.backup_id)
+
+            backup_info = catalog.get_backup_info(backup_id)
+            if not backup_info:
+                logging.error(
+                    "Backup %s for server %s does not exists",
+                    backup_id,
+                    config.server_name,
+                )
+                raise OperationErrorExit()
+
+            _validate_config(config, backup_info)
+
+            if backup_info.snapshots_info:
+                downloader = CloudBackupDownloaderSnapshot(cloud_interface, catalog)
+                downloader.download_backup(
+                    backup_info,
+                    config.recovery_dir,
+                    config.snapshot_recovery_instance,
+                    config.snapshot_recovery_zone,
+                )
+            else:
+                downloader = CloudBackupDownloaderObjectStore(cloud_interface, catalog)
+                downloader.download_backup(
+                    backup_info,
+                    config.recovery_dir,
+                    tablespace_map(config.tablespace),
+                )
 
     except KeyboardInterrupt as exc:
         logging.error("Barman cloud restore was interrupted by the user")
@@ -103,6 +149,14 @@ def parse_arguments(args=None):
         action="append",
         default=[],
     )
+    parser.add_argument(
+        "--snapshot-recovery-instance",
+        help="Instance where the disks recovered from the snapshots are attached",
+    )
+    parser.add_argument(
+        "--snapshot-recovery-zone",
+        help="Zone containing the instance and disks for the snapshot recovery",
+    )
     return parser.parse_args(args=args)
 
 
@@ -126,25 +180,25 @@ def tablespace_map(rules):
     return tablespaces
 
 
-class CloudBackupDownloader(object):
+class CloudBackupDownloader(with_metaclass(ABCMeta)):
     """
-    Cloud storage download client
+    Restore a backup from cloud storage.
     """
 
-    def __init__(self, cloud_interface, server_name):
+    def __init__(self, cloud_interface, catalog):
         """
         Object responsible for handling interactions with cloud storage
 
         :param CloudInterface cloud_interface: The interface to use to
           upload the backup
         :param str server_name: The name of the server as configured in Barman
+        :param CloudBackupCatalog catalog: The cloud backup catalog
         """
-
         self.cloud_interface = cloud_interface
-        self.server_name = server_name
-        self.catalog = CloudBackupCatalog(cloud_interface, server_name)
+        self.catalog = catalog
 
-    def download_backup(self, backup_id, destination_dir, tablespaces):
+    @abstractmethod
+    def download_backup(self, backup_id, destination_dir):
         """
         Download a backup from cloud storage
 
@@ -152,11 +206,23 @@ class CloudBackupDownloader(object):
         :param str destination_dir: Path to the destination directory
         """
 
-        backup_info = self.catalog.get_backup_info(backup_id)
 
-        if not backup_info:
+class CloudBackupDownloaderObjectStore(CloudBackupDownloader):
+    """
+    Cloud storage download client for an object store backup
+    """
+
+    def download_backup(self, backup_info, destination_dir, tablespaces):
+        """
+        Download a backup from cloud storage
+
+        :param BackupInfo backup_info: The backup info for the backup to restore
+        :param str destination_dir: Path to the destination directory
+        """
+        # Validate the destination directory before starting recovery
+        if os.path.exists(destination_dir) and os.listdir(destination_dir):
             logging.error(
-                "Backup %s for server %s does not exists", backup_id, self.server_name
+                "Destination %s already exists and it is not empty", destination_dir
             )
             raise OperationErrorExit()
 
@@ -234,5 +300,40 @@ class CloudBackupDownloader(object):
             os.mkdir(wal_path)
 
 
+class CloudBackupDownloaderSnapshot(CloudBackupDownloader):
+    """A minimal downloader for cloud backups which just retrieves the backup label."""
+
+    def download_backup(
+        self, backup_info, destination_dir, recovery_instance, recovery_zone
+    ):
+        """
+        Download a backup from cloud storage
+
+        :param BackupInfo backup_info: The backup info for the backup to restore
+        :param str destination_dir: Path to the destination directory
+        :param str recovery_instance: The name of the VM instance to which the disks
+            cloned from the backup snapshots are attached.
+        :param str recovery_zone: The zone in which the recovery disks and instance
+            reside.
+        """
+        snapshot_interface = get_snapshot_interface_from_backup_info(backup_info)
+        attached_snapshots = SnapshotRecoveryExecutor.get_attached_snapshots_for_backup(
+            snapshot_interface, backup_info, recovery_instance, recovery_zone
+        )
+        cmd = UnixLocalCommand()
+        SnapshotRecoveryExecutor.check_mount_points(
+            backup_info, attached_snapshots, cmd
+        )
+        SnapshotRecoveryExecutor.check_recovery_dir_exists(destination_dir, cmd)
+
+        # If the target directory does not exist then we will fail here because
+        # it tells us the snapshot has not been restored.
+        return self.cloud_interface.download_file(
+            "/".join((self.catalog.prefix, backup_info.backup_id, "backup_label")),
+            os.path.join(destination_dir, "backup_label"),
+            decompress=None,
+        )
+
+
 if __name__ == "__main__":
     main()
