@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2022
+# © Copyright EnterpriseDB UK Limited 2011-2023
 #
 # This file is part of Barman.
 #
@@ -219,7 +219,7 @@ class Command(object):
         if out_handler:
             self.out_handler = out_handler
         else:
-            self.out_handler = self.make_logging_handler(logging.INFO)
+            self.out_handler = self.make_logging_handler(logging.DEBUG)
         # If an error handler has been provided use it, otherwise log the
         # stderr as WARNING
         if err_handler:
