@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2022
+# © Copyright EnterpriseDB UK Limited 2011-2023
 #
 # This file is part of Barman.
 #
@@ -66,7 +66,7 @@ _SI_SUFFIX_RE = re.compile(r"""(\d+)\s*(k|Ki|M|Mi|G|Gi|T|Ti)?\s*$""")
 REUSE_BACKUP_VALUES = ("copy", "link", "off")
 
 # Possible copy methods for backups (must be all lowercase)
-BACKUP_METHOD_VALUES = ["rsync", "postgres", "local-rsync"]
+BACKUP_METHOD_VALUES = ["rsync", "postgres", "local-rsync", "snapshot"]
 
 CREATE_SLOT_VALUES = ["manual", "auto"]
 
@@ -384,6 +384,22 @@ def parse_slot_name(value):
     return value
 
 
+def parse_snapshot_disks(value):
+    """
+    Parse a comma separated list of names used to reference disks managed by a cloud
+    provider.
+
+    :param str value: Comma separated list of disk names
+    :return: List of disk names
+    """
+    disk_names = value.split(",")
+    # Verify each parsed disk is not an empty string
+    for disk_name in disk_names:
+        if disk_name == "":
+            raise ValueError(disk_names)
+    return disk_names
+
+
 def parse_create_slot(value):
     """
     Parse a string to a valid create_slot value.
@@ -474,6 +490,11 @@ class ServerConfig(object):
         "retention_policy_mode",
         "reuse_backup",
         "slot_name",
+        "snapshot_disks",
+        "snapshot_gcp_project",
+        "snapshot_instance",
+        "snapshot_provider",
+        "snapshot_zone",
         "ssh_command",
         "streaming_archiver",
         "streaming_archiver_batch_size",
@@ -543,6 +564,8 @@ class ServerConfig(object):
         "retention_policy_mode",
         "reuse_backup",
         "slot_name",
+        "snapshot_gcp_project",
+        "snapshot_provider",
         "streaming_archiver",
         "streaming_archiver_batch_size",
         "streaming_archiver_name",
@@ -614,6 +637,7 @@ class ServerConfig(object):
         "recovery_staging_path": parse_recovery_staging_path,
         "create_slot": parse_create_slot,
         "reuse_backup": parse_reuse_backup,
+        "snapshot_disks": parse_snapshot_disks,
         "streaming_archiver": parse_boolean,
         "streaming_archiver_batch_size": int,
         "slot_name": parse_slot_name,
