@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2022
+# © Copyright EnterpriseDB UK Limited 2011-2023
 #
 # This file is part of Barman.
 #
@@ -116,6 +116,18 @@ class BackupInfoBadInitialisation(BackupException):
     """
 
 
+class BackupPreconditionException(BackupException):
+    """
+    Exception for a backup precondition not being met
+    """
+
+
+class SnapshotBackupException(BackupException):
+    """
+    Exception for snapshot backups
+    """
+
+
 class SyncError(SyncException):
     """
     Synchronisation error
@@ -358,6 +370,12 @@ class RecoveryException(BarmanException):
     """
 
 
+class RecoveryPreconditionException(RecoveryException):
+    """
+    Exception for a recovery precondition not being met
+    """
+
+
 class RecoveryTargetActionException(RecoveryException):
     """
     Exception for a wrong recovery target action
