@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2013-2022
+# © Copyright EnterpriseDB UK Limited 2013-2023
 #
 # This file is part of Barman.
 #
@@ -26,6 +26,7 @@ import dateutil.parser
 import dateutil.tz
 
 from barman import xlog
+from barman.cloud_providers import snapshots_info_from_dict
 from barman.exceptions import BackupInfoBadInitialisation
 from barman.utils import fsync_dir
 
@@ -36,6 +37,16 @@ Tablespace = collections.namedtuple("Tablespace", "name oid location")
 # Named tuple representing a file 'path' with an associated 'file_type'
 TypedFile = collections.namedtuple("ConfFile", "file_type path")
 
+
+def output_snapshots_info(snapshots_info):
+    return null_repr(snapshots_info.to_dict())
+
+
+def load_snapshots_info(string):
+    obj = ast.literal_eval(string)
+    return snapshots_info_from_dict(obj)
+
+
 _logger = logging.getLogger(__name__)
 
 
@@ -153,6 +164,13 @@ class FieldListFile(object):
 
     __slots__ = ("_fields", "filename")
 
+    # A list of fields which should be hidden if they are not set.
+    # Such fields will not be written to backup.info files or included in the
+    # backup.info items unles they are set to a non-None value.
+    # Any fields listed here should be removed from the list at the next major
+    # version increase.
+    _hide_if_null = ()
+
     def __init__(self, **kwargs):
         """
         Represent a predefined set of keys with the associated value.
@@ -222,6 +240,8 @@ class FieldListFile(object):
         try:
             for name, field in sorted(inspect.getmembers(type(self))):
                 value = getattr(self, name, None)
+                if value is None and name in self._hide_if_null:
+                    continue
                 if isinstance(field, Field):
                     if callable(field.to_str):
                         value = field.to_str(value)
@@ -300,6 +320,8 @@ class FieldListFile(object):
         """
         for name, field in sorted(inspect.getmembers(type(self))):
             value = getattr(self, name, None)
+            if value is None and name in self._hide_if_null:
+                continue
             if isinstance(field, Field):
                 if callable(field.to_str):
                     value = field.to_str(value)
@@ -472,9 +494,15 @@ class BackupInfo(FieldListFile):
     )
     systemid = Field("systemid")
     compression = Field("compression")
+    backup_name = Field("backup_name")
+    snapshots_info = Field(
+        "snapshots_info", load=load_snapshots_info, dump=output_snapshots_info
+    )
 
     __slots__ = "backup_id", "backup_version"
 
+    _hide_if_null = ("backup_name", "snapshots_info")
+
     def __init__(self, backup_id, **kwargs):
         """
         Stores meta information about a single backup
@@ -538,14 +566,21 @@ class BackupInfo(FieldListFile):
         :return dict:
         """
         result = dict(self.items())
-        result.update(
-            backup_id=self.backup_id,
-            server_name=self.server_name,
-            mode=self.mode,
-            tablespaces=self.tablespaces,
-            included_files=self.included_files,
-            copy_stats=self.copy_stats,
+        top_level_fields = (
+            "backup_id",
+            "server_name",
+            "mode",
+            "tablespaces",
+            "included_files",
+            "copy_stats",
+            "snapshots_info",
         )
+        for field_name in top_level_fields:
+            field_value = getattr(self, field_name)
+            if field_value is not None or field_name not in self._hide_if_null:
+                result.update({field_name: field_value})
+        if self.snapshots_info is not None:
+            result.update({"snapshots_info": self.snapshots_info.to_dict()})
         return result
 
     def to_json(self):
