@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2022
+# © Copyright EnterpriseDB UK Limited 2011-2023
 #
 # This file is part of Barman.
 #
@@ -54,7 +54,6 @@ from barman.infofile import Tablespace
 from barman.postgres_plumbing import function_name_map
 from barman.remote_status import RemoteStatusMixin
 from barman.utils import force_str, simplify_version, with_metaclass
-from barman.xlog import DEFAULT_XLOG_SEG_SIZE
 
 # This is necessary because the CONFIGURATION_LIMIT_EXCEEDED constant
 # has been added in psycopg2 2.5, but Barman supports version 2.4.2+ so
@@ -93,6 +92,7 @@ class PostgreSQL(with_metaclass(ABCMeta, RemoteStatusMixin)):
     """
 
     CHECK_QUERY = "SELECT 1"
+    MINIMAL_VERSION = 90600
 
     def __init__(self, conninfo):
         """
@@ -249,24 +249,43 @@ class PostgreSQL(with_metaclass(ABCMeta, RemoteStatusMixin)):
         """
         try:
             conn = self.connect()
-            major = int(conn.server_version / 10000)
-            minor = int(conn.server_version / 100 % 100)
-            patch = int(conn.server_version % 100)
-            if major < 10:
-                return "%d.%d.%d" % (major, minor, patch)
-            if minor != 0:
-                _logger.warning(
-                    "Unexpected non zero minor version %s in %s",
-                    minor,
-                    conn.server_version,
-                )
-            return "%d.%d" % (major, patch)
+            return self.int_version_to_string_version(conn.server_version)
+
         except PostgresConnectionError as e:
             _logger.debug(
                 "Error retrieving PostgreSQL version: %s", force_str(e).strip()
             )
             return None
 
+    @property
+    def minimal_txt_version(self):
+        """
+        Human readable version of PostgreSQL (calculated from server_version)
+
+        :rtype: str|None
+        """
+        return self.int_version_to_string_version(self.MINIMAL_VERSION)
+
+    @staticmethod
+    def int_version_to_string_version(int_version):
+        """
+        takes an int version
+        :param int_version: ex: 10.22 121200 or 130800
+        :return: str  ex 10.22.00 12.12.00  13.8.00
+        """
+        major = int(int_version / 10000)
+        minor = int(int_version / 100 % 100)
+        patch = int(int_version % 100)
+        if major < 10:
+            return "%d.%d.%d" % (major, minor, patch)
+        if minor != 0:
+            _logger.warning(
+                "Unexpected non zero minor version %s in %s",
+                minor,
+                int_version,
+            )
+        return "%d.%d" % (major, patch)
+
     @property
     def server_major_version(self):
         """
@@ -279,6 +298,10 @@ class PostgreSQL(with_metaclass(ABCMeta, RemoteStatusMixin)):
             return simplify_version(result)
         return None
 
+    def is_minimal_postgres_version(self):
+        """Checks if postgres version has at least minimal version"""
+        return self.server_version >= self.MINIMAL_VERSION
+
 
 class StreamingConnection(PostgreSQL):
     """
@@ -337,17 +360,17 @@ class StreamingConnection(PostgreSQL):
                 "streaming_systemid",
                 "timeline",
                 "xlogpos",
+                "version_supported",
             ),
             None,
         )
         try:
-            # If the server is too old to support `pg_receivexlog`,
-            # exit immediately.
             # This needs to be protected by the try/except because
-            # `self.server_version` can raise a PostgresConnectionError
-            if self.server_version < 90200:
-                result["streaming_supported"] = False
+            # `self.is_minimal_postgres_version` can raise a PostgresConnectionError
+            result["version_supported"] = self.is_minimal_postgres_version()
+            if not self.is_minimal_postgres_version():
                 return result
+            # streaming is always supported
             result["streaming_supported"] = True
             # Execute a IDENTIFY_SYSTEM to check the connection
             cursor = self._cursor()
@@ -462,9 +485,7 @@ class PostgreSQLConnection(PostgreSQL):
             return self._conn
 
         self._conn = super(PostgreSQLConnection, self).connect()
-        server_version = self._conn.server_version
-        use_app_name = "application_name" in self.conn_parameters
-        if server_version >= 90000 and not use_app_name:
+        if "application_name" not in self.conn_parameters:
             try:
                 cur = self._conn.cursor()
                 # Do not use parameter substitution with SET
@@ -507,36 +528,12 @@ class PostgreSQLConnection(PostgreSQL):
             )
             return None
 
-    @property
-    def has_pgespresso(self):
-        """
-        Returns true if the `pgespresso` extension is available
-        """
-        try:
-            # pg_extension is only available from Postgres 9.1+
-            if self.server_version < 90100:
-                return False
-            cur = self._cursor()
-            cur.execute(
-                "SELECT count(*) FROM pg_extension WHERE extname = 'pgespresso'"
-            )
-            q_result = cur.fetchone()[0]
-            return q_result > 0
-        except (PostgresConnectionError, psycopg2.Error) as e:
-            _logger.debug(
-                "Error retrieving pgespresso information: %s", force_str(e).strip()
-            )
-            return None
-
     @property
     def is_in_recovery(self):
         """
         Returns true if PostgreSQL server is in recovery mode (hot standby)
         """
         try:
-            # pg_is_in_recovery is only available from Postgres 9.0+
-            if self.server_version < 90000:
-                return False
             cur = self._cursor()
             cur.execute("SELECT pg_is_in_recovery()")
             return cur.fetchone()[0]
@@ -709,11 +706,6 @@ class PostgreSQLConnection(PostgreSQL):
         :return: The wal size (In bytes)
         """
 
-        # Prior to PostgreSQL 8.4, the wal segment size was not configurable,
-        # even in compilation
-        if self.server_version < 80400:
-            return DEFAULT_XLOG_SEG_SIZE
-
         try:
             cur = self._cursor(cursor_factory=DictCursor)
             # We can't use the `get_setting` method here, because it
@@ -828,9 +820,6 @@ class PostgreSQLConnection(PostgreSQL):
             pg_stat_archiver or None
         """
         try:
-            # pg_stat_archiver is only available from Postgres 9.4+
-            if self.server_version < 90400:
-                return None
             cur = self._cursor(cursor_factory=DictCursor)
             # Select from pg_stat_archiver statistics view,
             # retrieving statistics about WAL archiver process activity,
@@ -880,11 +869,11 @@ class PostgreSQLConnection(PostgreSQL):
             "is_superuser",
             "is_in_recovery",
             "current_xlog",
-            "pgespresso_installed",
             "replication_slot_support",
             "replication_slot",
             "synchronous_standby_names",
             "postgres_systemid",
+            "version_supported",
         ]
         # Initialise the result dictionary setting all the values to None
         result = dict.fromkeys(
@@ -893,25 +882,22 @@ class PostgreSQLConnection(PostgreSQL):
         try:
             # Retrieve wal_level, hot_standby and max_wal_senders
             # only if version is >= 9.0
-            if self.server_version >= 90000:
-                pg_settings.append("wal_level")
-                pg_settings.append("hot_standby")
-                pg_settings.append("max_wal_senders")
-                # Retrieve wal_keep_segments from version 9.0 onwards, until
-                # version 13.0, where it was renamed to wal_keep_size
-                if self.server_version < 130000:
-                    pg_settings.append("wal_keep_segments")
-                else:
-                    pg_settings.append("wal_keep_size")
-
-            if self.server_version >= 90300:
-                pg_settings.append("data_checksums")
-
-            if self.server_version >= 90400:
-                pg_settings.append("max_replication_slots")
-
-            if self.server_version >= 90500:
-                pg_settings.append("wal_compression")
+            pg_settings.extend(
+                [
+                    "wal_level",
+                    "hot_standby",
+                    "max_wal_senders",
+                    "data_checksums",
+                    "max_replication_slots",
+                    "wal_compression",
+                ]
+            )
+            # Retrieve wal_keep_segments from version 9.0 onwards, until
+            # version 13.0, where it was renamed to wal_keep_size
+            if self.server_version < 130000:
+                pg_settings.append("wal_keep_segments")
+            else:
+                pg_settings.append("wal_keep_size")
 
             # retrieves superuser settings
             if self.has_backup_privileges:
@@ -926,7 +912,7 @@ class PostgreSQLConnection(PostgreSQL):
             result["has_backup_privileges"] = self.has_backup_privileges
             result["is_in_recovery"] = self.is_in_recovery
             result["server_txt_version"] = self.server_txt_version
-            result["pgespresso_installed"] = self.has_pgespresso
+            result["version_supported"] = self.is_minimal_postgres_version()
             current_xlog_info = self.current_xlog_info
             if current_xlog_info:
                 result["current_lsn"] = current_xlog_info["location"]
@@ -942,23 +928,13 @@ class PostgreSQLConnection(PostgreSQL):
             result.update(self.get_configuration_files())
 
             # Retrieve the replication_slot status
-            result["replication_slot_support"] = False
-            if self.server_version >= 90400:
-                result["replication_slot_support"] = True
-                if self.slot_name is not None:
-                    result["replication_slot"] = self.get_replication_slot(
-                        self.slot_name
-                    )
+            result["replication_slot_support"] = True
+            if self.slot_name is not None:
+                result["replication_slot"] = self.get_replication_slot(self.slot_name)
 
             # Retrieve the list of synchronous standby names
-            result["synchronous_standby_names"] = []
-            if self.server_version >= 90100:
-                result[
-                    "synchronous_standby_names"
-                ] = self.get_synchronous_standby_names()
-
-            if self.server_version >= 90600:
-                result["postgres_systemid"] = self.get_systemid()
+            result["synchronous_standby_names"] = self.get_synchronous_standby_names()
+            result["postgres_systemid"] = self.get_systemid()
         except (PostgresConnectionError, psycopg2.Error) as e:
             _logger.warning(
                 "Error retrieving PostgreSQL status: %s", force_str(e).strip()
@@ -969,9 +945,6 @@ class PostgreSQLConnection(PostgreSQL):
         """
         Get a Postgres instance systemid
         """
-        if self.server_version < 90600:
-            return
-
         try:
             cur = self._cursor()
             cur.execute("SELECT system_identifier::text FROM pg_control_system()")
@@ -1006,18 +979,14 @@ class PostgreSQLConnection(PostgreSQL):
         """
         try:
             cur = self._cursor()
-            if self.server_version >= 90200:
-                cur.execute(
-                    "SELECT spcname, oid, "
-                    "pg_tablespace_location(oid) AS spclocation "
-                    "FROM pg_tablespace "
-                    "WHERE pg_tablespace_location(oid) != ''"
-                )
-            else:
-                cur.execute(
-                    "SELECT spcname, oid, spclocation "
-                    "FROM pg_tablespace WHERE spclocation != ''"
-                )
+
+            cur.execute(
+                "SELECT spcname, oid, "
+                "pg_tablespace_location(oid) AS spclocation "
+                "FROM pg_tablespace "
+                "WHERE pg_tablespace_location(oid) != ''"
+            )
+
             # Generate a list of tablespace objects
             return [Tablespace._make(item) for item in cur.fetchall()]
         except (PostgresConnectionError, psycopg2.Error) as e:
@@ -1046,21 +1015,20 @@ class PostgreSQLConnection(PostgreSQL):
                 self.configuration_files[cname] = cpath
 
             # Retrieve additional configuration files
-            # If PostgreSQL is older than 8.4 disable this check
-            if self.server_version >= 80400:
-                cur.execute(
-                    "SELECT DISTINCT sourcefile AS included_file "
-                    "FROM pg_settings "
-                    "WHERE sourcefile IS NOT NULL "
-                    "AND sourcefile NOT IN "
-                    "(SELECT setting FROM pg_settings "
-                    "WHERE name = 'config_file') "
-                    "ORDER BY 1"
-                )
-                # Extract the values from the containing single element tuples
-                included_files = [included_file for included_file, in cur.fetchall()]
-                if len(included_files) > 0:
-                    self.configuration_files["included_files"] = included_files
+
+            cur.execute(
+                "SELECT DISTINCT sourcefile AS included_file "
+                "FROM pg_settings "
+                "WHERE sourcefile IS NOT NULL "
+                "AND sourcefile NOT IN "
+                "(SELECT setting FROM pg_settings "
+                "WHERE name = 'config_file') "
+                "ORDER BY 1"
+            )
+            # Extract the values from the containing single element tuples
+            included_files = [included_file for included_file, in cur.fetchall()]
+            if len(included_files) > 0:
+                self.configuration_files["included_files"] = included_files
 
         except (PostgresConnectionError, psycopg2.Error) as e:
             _logger.debug(
@@ -1086,9 +1054,6 @@ class PostgreSQLConnection(PostgreSQL):
         :returns: the restore point LSN
         :rtype: str|None
         """
-        if self.server_version < 90100:
-            return None
-
         # Not possible if on a standby
         # Called inside the pg_connect context to reuse the connection
         if self.is_in_recovery:
@@ -1131,14 +1096,6 @@ class PostgreSQLConnection(PostgreSQL):
             cur = conn.cursor(cursor_factory=DictCursor)
             if self.server_version >= 150000:
                 raise PostgresObsoleteFeature("15")
-            elif self.server_version < 80400:
-                cur.execute(
-                    "SELECT location, "
-                    "({pg_walfile_name_offset}(location)).*, "
-                    "now() AS timestamp "
-                    "FROM {pg_backup_start}(%s) AS location".format(**self.name_map),
-                    (label,),
-                )
             else:
                 cur.execute(
                     "SELECT location, "
@@ -1315,81 +1272,6 @@ class PostgreSQLConnection(PostgreSQL):
             _logger.debug(msg)
             raise PostgresException(msg)
 
-    def pgespresso_start_backup(self, label):
-        """
-        Execute a pgespresso_start_backup
-
-        This method returns a dictionary containing the following data:
-
-         * backup_label
-         * timestamp
-
-        :param str label: descriptive string to identify the backup
-        :rtype: psycopg2.extras.DictRow
-        """
-        try:
-            conn = self.connect()
-
-            # Rollback to release the transaction,
-            # as the pgespresso_start_backup invocation can last
-            # up to PostgreSQL's checkpoint_timeout
-            conn.rollback()
-
-            # Start the concurrent backup using pgespresso
-            cur = conn.cursor(cursor_factory=DictCursor)
-            cur.execute(
-                "SELECT pgespresso_start_backup(%s,%s) AS backup_label, "
-                "now() AS timestamp",
-                (label, self.immediate_checkpoint),
-            )
-
-            start_row = cur.fetchone()
-
-            # Rollback to release the transaction, as the connection
-            # is to be retained until the end of backup
-            conn.rollback()
-
-            return start_row
-        except (PostgresConnectionError, psycopg2.Error) as e:
-            msg = "pgespresso_start_backup(): %s" % force_str(e).strip()
-            _logger.debug(msg)
-            raise PostgresException(msg)
-
-    def pgespresso_stop_backup(self, backup_label):
-        """
-        Execute a pgespresso_stop_backup
-
-        This method returns a dictionary containing the following data:
-
-         * end_wal
-         * timestamp
-
-        :param str backup_label: backup label as returned
-            by pgespresso_start_backup
-        :rtype: psycopg2.extras.DictRow
-        """
-        try:
-            conn = self.connect()
-            # Issue a rollback to release any unneeded lock
-            conn.rollback()
-            cur = conn.cursor(cursor_factory=DictCursor)
-            cur.execute(
-                "SELECT pgespresso_stop_backup(%s) AS end_wal, now() AS timestamp",
-                (backup_label,),
-            )
-            return cur.fetchone()
-        except (PostgresConnectionError, psycopg2.Error) as e:
-            msg = "Error issuing pgespresso_stop_backup() command: %s" % (
-                force_str(e).strip()
-            )
-            _logger.debug(msg)
-            raise PostgresException(
-                "%s\n"
-                "HINT: You might have to manually execute "
-                "pgespresso_abort_backup() on your PostgreSQL "
-                "server" % msg
-            )
-
     def switch_wal(self):
         """
         Execute a pg_switch_wal()
@@ -1498,9 +1380,6 @@ class PostgreSQLConnection(PostgreSQL):
             # - sync_state
             #
 
-            if self.server_version < 90100:
-                raise PostgresUnsupportedFeature("9.1")
-
             from_repslot = ""
             where_clauses = []
             if self.server_version >= 100000:
@@ -1511,7 +1390,7 @@ class PostgreSQLConnection(PostgreSQL):
                     "LEFT JOIN pg_replication_slots rs ON (r.pid = rs.active_pid) "
                 )
                 where_clauses += ["(rs.slot_type IS NULL OR rs.slot_type = 'physical')"]
-            elif self.server_version >= 90500:
+            else:
                 # PostgreSQL 9.5/9.6
                 what = (
                     "pid, "
@@ -1537,66 +1416,6 @@ class PostgreSQLConnection(PostgreSQL):
                     "LEFT JOIN pg_replication_slots rs ON (r.pid = rs.active_pid) "
                 )
                 where_clauses += ["(rs.slot_type IS NULL OR rs.slot_type = 'physical')"]
-            elif self.server_version >= 90400:
-                # PostgreSQL 9.4
-                what = (
-                    "pid, "
-                    "usesysid, "
-                    "usename, "
-                    "application_name, "
-                    "client_addr, "
-                    "client_hostname, "
-                    "client_port, "
-                    "backend_start, "
-                    "backend_xmin, "
-                    "state, "
-                    "sent_location AS sent_lsn, "
-                    "write_location AS write_lsn, "
-                    "flush_location AS flush_lsn, "
-                    "replay_location AS replay_lsn, "
-                    "sync_priority, "
-                    "sync_state"
-                )
-            elif self.server_version >= 90200:
-                # PostgreSQL 9.2/9.3
-                what = (
-                    "pid, "
-                    "usesysid, "
-                    "usename, "
-                    "application_name, "
-                    "client_addr, "
-                    "client_hostname, "
-                    "client_port, "
-                    "backend_start, "
-                    "CAST (NULL AS xid) AS backend_xmin, "
-                    "state, "
-                    "sent_location AS sent_lsn, "
-                    "write_location AS write_lsn, "
-                    "flush_location AS flush_lsn, "
-                    "replay_location AS replay_lsn, "
-                    "sync_priority, "
-                    "sync_state"
-                )
-            else:
-                # PostgreSQL 9.1
-                what = (
-                    "procpid AS pid, "
-                    "usesysid, "
-                    "usename, "
-                    "application_name, "
-                    "client_addr, "
-                    "client_hostname, "
-                    "client_port, "
-                    "backend_start, "
-                    "CAST (NULL AS xid) AS backend_xmin, "
-                    "state, "
-                    "sent_location AS sent_lsn, "
-                    "write_location AS write_lsn, "
-                    "flush_location AS flush_lsn, "
-                    "replay_location AS replay_lsn, "
-                    "sync_priority, "
-                    "sync_state"
-                )
 
             # Streaming client
             if client_type == self.STANDBY:
@@ -1744,7 +1563,7 @@ class StandbyPostgreSQLConnection(PostgreSQLConnection):
     def __init__(
         self,
         conninfo,
-        primary_conn,
+        primary_conninfo,
         immediate_checkpoint=False,
         slot_name=None,
         application_name="barman",
@@ -1753,7 +1572,7 @@ class StandbyPostgreSQLConnection(PostgreSQLConnection):
         Standby PostgreSQL connection constructor.
 
         :param str conninfo: Connection information (aka DSN) for the standby.
-        :param barman.postgres.PostgreSQLConnection primary_conn: Connection to the
+        :param str primary_conninfo: Connection information (aka DSN) for the
             primary.
         :param bool immediate_checkpoint: Whether to do an immediate checkpoint
             when a backup is started.
@@ -1768,7 +1587,10 @@ class StandbyPostgreSQLConnection(PostgreSQLConnection):
         )
         # The standby connection has its own connection object used to talk to the
         # primary when switching WALs.
-        self.primary = primary_conn
+        self.primary_conninfo = primary_conninfo
+        # The standby needs a connection to the primary so that it can
+        # perform WAL switches itself when calling pg_backup_stop.
+        self.primary = PostgreSQLConnection(self.primary_conninfo)
 
     def close(self):
         """Close the connection to PostgreSQL."""
@@ -1797,6 +1619,9 @@ class StandbyPostgreSQLConnection(PostgreSQLConnection):
             performed.
         :param int wait: The number of seconds to wait between WAL switches.
         """
+        # Use a new connection to prevent undefined behaviour
+        self.primary = PostgreSQLConnection(self.primary_conninfo)
+
         # The stop backup call on the standby may have already completed by this
         # point so check whether we have been told to stop.
         try:
@@ -1804,19 +1629,23 @@ class StandbyPostgreSQLConnection(PostgreSQLConnection):
                 return
         except Empty:
             pass
-        # Start calling pg_switch_wal on the primary until we either read something
-        # from the done queue or we exceed the number of WAL switches we are allowed.
-        for _ in range(0, times):
-            self.switch_wal()
-            # See if we have been told to stop. We use the wait value as our timeout
-            # so that we can exit immediately if we receive a stop message or proceed
-            # to another WAL switch if the wait time is exceeded.
-            try:
-                if done_q.get(timeout=wait):
-                    return
-            except Empty:
-                # An empty queue just means we haven't yet been told to stop
-                pass
+        try:
+            # Start calling pg_switch_wal on the primary until we either read something
+            # from the done queue or we exceed the number of WAL switches we are allowed.
+            for _ in range(0, times):
+                self.switch_wal()
+                # See if we have been told to stop. We use the wait value as our timeout
+                # so that we can exit immediately if we receive a stop message or proceed
+                # to another WAL switch if the wait time is exceeded.
+                try:
+                    if done_q.get(timeout=wait):
+                        return
+                except Empty:
+                    # An empty queue just means we haven't yet been told to stop
+                    pass
+        finally:
+            # Close the connection since only this subprocess will ever use it
+            self.primary.close()
 
     def _start_wal_switch(self):
         """Start switching WALs in a child process."""
